#
# Copyright (c) 2015, 2015, Oracle and/or its affiliates. All rights reserved.
# DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
#
# This code is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 only, as
# published by the Free Software Foundation.
#
# This code is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# version 2 for more details (a copy is included in the LICENSE file that
# accompanied this code).
#
# You should have received a copy of the GNU General Public License version
# 2 along with this work; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
# or visit www.oracle.com if you need additional information or have any
# questions.
#

fastr.createcc <- function(func) .FastR(.NAME="createcc", func)

fastr.getcc <- function(func) .FastR(.NAME="getcc", func)

fastr.compile <- function(func, background=TRUE) .FastR(.NAME="compile", func, background)

fastr.dumptrees <- function(func, igvDump=FALSE, verbose=FALSE) .FastR(.NAME="dumptrees", func, igvDump, verbose)

fastr.source <- function(func) .FastR(.NAME="source", func)

fastr.syntaxtree <- function(func, source=FALSE) .FastR(.NAME="syntaxtree", func, source)

fastr.tree <- function(func, verbose=FALSE) .FastR(.NAME="tree", func, verbose)

fastr.typeof <- function(x) .FastR(.NAME="typeof", x)

fastr.stacktrace <- function(print.frame.contents=TRUE) .FastR(.NAME="stacktrace", print.frame.contents)

fastr.debug <- function(element) .FastR(.NAME="debug", element)

fastr.inspect <- function(...) .FastR(.NAME="inspect", ...)

fastr.createpkgsources <- function(pkgs = NULL) {
    if (!length(pkgs)) {
        pkgs <- sub("package:", "", grep("package:", search(), value=TRUE, fixed=TRUE), fixed=TRUE)
    }
    for (pkg in pkgs) {
    	ns <- asNamespace(pkg)
    	for (n in ls(envir=ns, all.names=TRUE)) {
    	    .FastR(.NAME="pkgsource.pre", pkg, n)
    	     val <- get(n, ns)
    	    .FastR(.NAME="pkgsource.post", pkg, n, val)
    	}
    }
	.FastR(.NAME="pkgsource.done", pkg)
	invisible(NULL)
}

fastr.createpkgsource <- function(pkg, name) {
	ns <- asNamespace(pkg)
	n <- ls(pattern=name, envir=ns, all.names=TRUE)
	.FastR(.NAME="pkgsource.pre", pkg, n)
	val <- get(n, ns)
	.FastR(.NAME="pkgsource.post", pkg, n, val)
	.FastR(.NAME="pkgsource.done", pkg)
	invisible(NULL)
}

fastr.comparefilesizes <- function(file1, file2) .FastR(.NAME="comparefilesizes", file1, file2)

fastr_context.create <- function(args="", kind="SHARED_NOTHING") {
	kind <- match(kind, c("SHARED_NOTHING", "SHARED_PACKAGES", "SHARED_CODE"))
	if (is.na(kind)) stop("invalid kind argument")
	context <- .FastR(.NAME="context.create", args, kind)
	class(context) <- "fastr_context"
    context
}

print.fastr_context <- function(context, ...) {
	.FastR(.NAME="context.print", context)
	invisible(context)
}

fastr_context.eval <- function(contexts, exprs, par=FALSE) {
	.FastR(.NAME="context.eval", contexts, exprs, par)
	invisible(NULL)
}

fastr_context.pareval <- function(contexts, exprs) {
	fastr_context.eval(contexts, exprs, par=TRUE)
}



